@@ -31,8 +31,6 @@ from model_card_toolkit.utils import validation
 _SCHEMA_VERSION_STRING = "schema_version"
 
 
-# TODO(b/181702622): Think about a smart and clean way to control the required
-# field.
 @dataclasses.dataclass
 class Owner(BaseModelCardField):
   """The information about owners of a model.
@@ -136,6 +134,7 @@ class ModelDetails(BaseModelCardField):
       may be useful to your audience.
     citations: How should the model be cited? If the model is based on published
       academic research, cite the research.
+    path: The path where the model is stored.
   """
   name: Optional[Text] = None
   overview: Optional[Text] = None
@@ -145,6 +144,7 @@ class ModelDetails(BaseModelCardField):
   licenses: List[License] = dataclasses.field(default_factory=list)
   references: List[Reference] = dataclasses.field(default_factory=list)
   citations: List[Citation] = dataclasses.field(default_factory=list)
+  path: Optional[Text] = None
 
   _proto_type: dataclasses.InitVar[type(
       model_card_pb2.ModelDetails)] = model_card_pb2.ModelDetails
@@ -241,6 +241,21 @@ class Dataset(BaseModelCardField):
       model_card_pb2.Dataset)] = model_card_pb2.Dataset
 
 
+@dataclasses.dataclass
+class KeyVal(BaseModelCardField):
+  """A generic key-value pair.
+
+  Attributes:
+    key: The key of the key-value pair.
+    value: The value of the key-value pair.
+  """
+  key: Optional[Text] = None
+  value: Optional[Text] = None
+
+  _proto_type: dataclasses.InitVar[type(
+      model_card_pb2.KeyVal)] = model_card_pb2.KeyVal
+
+
 @dataclasses.dataclass
 class ModelParameters(BaseModelCardField):
   """Parameters for construction of the model.
@@ -249,17 +264,38 @@ class ModelParameters(BaseModelCardField):
     model_architecture: specifies the architecture of your model.
     data: specifies the datasets used to train and evaluate your model.
     input_format: describes the data format for inputs to your model.
+    input_format_map: describes the data format for inputs to your model, in
+      key-value format.
     output_format: describes the data format for outputs from your model.
+    output_format_map: describes the data format for outputs to your model, in
+      key-value format
   """
   model_architecture: Optional[Text] = None
   data: List[Dataset] = dataclasses.field(default_factory=list)
   input_format: Optional[Text] = None
+  input_format_map: List[KeyVal] = dataclasses.field(default_factory=list)
   output_format: Optional[Text] = None
+  output_format_map: List[KeyVal] = dataclasses.field(default_factory=list)
 
   _proto_type: dataclasses.InitVar[type(
       model_card_pb2.ModelParameters)] = model_card_pb2.ModelParameters
 
 
+@dataclasses.dataclass
+class ConfidenceInterval(BaseModelCardField):
+  """The confidence interval of the metric.
+
+  Attributes:
+    lower_bound: The lower bound of the performance metric.
+    upper_bound: The upper bound of the performance metric.
+  """
+  lower_bound: Optional[Text] = None
+  upper_bound: Optional[Text] = None
+
+  _proto_type: dataclasses.InitVar[BaseModelCardField._get_type(
+      model_card_pb2.ConfidenceInterval)] = model_card_pb2.ConfidenceInterval
+
+
 @dataclasses.dataclass
 class PerformanceMetric(BaseModelCardField):
   """The details of the performance metric.
@@ -268,13 +304,13 @@ class PerformanceMetric(BaseModelCardField):
     type: What performance metric are you reporting on?
     value: What is the value of this performance metric?
     slice: What slice of your data was this metric computed on?
+    confidence_interval: The confidence interval of the metric.
   """
-  # TODO(b/179415408): add fields (name, value, confidence_interval, threshold,
-  # slice) after gathering requirements (potential clients: Jigsaw)
-  # The following fields are EXPERIMENTAL and introduced for migration purpose.
   type: Optional[Text] = None
   value: Optional[Text] = None
   slice: Optional[Text] = None
+  confidence_interval: ConfidenceInterval = dataclasses.field(
+      default_factory=ConfidenceInterval)
 
   _proto_type: dataclasses.InitVar[BaseModelCardField._get_type(
       model_card_pb2.PerformanceMetric)] = model_card_pb2.PerformanceMetric
@@ -445,22 +481,16 @@ class ModelCard(BaseModelCardField):
         _SCHEMA_VERSION_STRING] = validation.get_latest_schema_version()
     return json_lib.dumps(model_card_dict, indent=2)
 
-  def _from_json(self, json_dict: Dict[Text, Any]) -> "ModelCard":
-    """Read ModelCard from JSON.
+  def from_json(self, json_dict: Dict[Text, Any]) -> None:
+    """Reads ModelCard from JSON.
 
     If ModelCard fields have already been set, this function will overwrite any
     existing values.
 
-    WARNING: This method's interface may change in the future, do not use for
-    critical workflows.
-
     Args:
       json_dict: A JSON dict from which to populate fields in the model card
         schema.
 
-    Returns:
-      self
-
     Raises:
       JSONDecodeError: If `json_dict` is not a valid JSON string.
       ValidationError: If `json_dict` does not follow the model card JSON
@@ -497,4 +527,3 @@ class ModelCard(BaseModelCardField):
     validation.validate_json_schema(json_dict)
     self.clear()
     _populate_from_json(json_dict, self)
-    return self
