@@ -15,7 +15,7 @@
 
 import enum
 import os
-from typing import Iterable, List, Optional, Text, Union
+from typing import Any, Dict, Iterable, List, Optional, Sequence, Text, Union
 
 from absl import logging
 import attr
@@ -34,6 +34,15 @@ _TFX_MODEL_TYPE = 'Model'
 _TFX_METRICS_TYPE = 'ModelEvaluation'
 _TFX_TRAINER_TYPE = 'tfx.components.trainer.component.Trainer'
 
+# Map of data types to field names in a TFMA arrayValue
+_TYPE_FIELD_MAP = {
+    'BYTES': 'bytesValues',
+    'INT32': 'int32Values',
+    'INT64': 'int64Values',
+    'FLOAT32': 'float32Values',
+    'FLOAT64': 'float64Values'
+}
+
 
 @attr.s(auto_attribs=True)
 class PipelineTypes(object):
@@ -47,8 +56,7 @@ class PipelineTypes(object):
   trainer_type: metadata_store_pb2.ExecutionType
 
 
-def _get_tfx_pipeline_types(
-    store: mlmd.MetadataStore) -> PipelineTypes:
+def _get_tfx_pipeline_types(store: mlmd.MetadataStore) -> PipelineTypes:
   """Retrieves the registered types in the given `store`.
 
   Args:
@@ -68,8 +76,7 @@ def _get_tfx_pipeline_types(
   missing_types = expected_artifact_types.difference(artifact_types.keys())
   if missing_types:
     raise ValueError(
-        f'Given `store` is invalid: missing ArtifactTypes: {missing_types}.'
-    )
+        f'Given `store` is invalid: missing ArtifactTypes: {missing_types}.')
   execution_types = {etype.name: etype for etype in store.get_execution_types()}
   expected_execution_types = {_TFX_TRAINER_TYPE}
   missing_types = expected_execution_types.difference(execution_types.keys())
@@ -274,8 +281,8 @@ def get_stats_artifacts_for_model(
 def _property_value(
     node: Union[metadata_store_pb2.Artifact, metadata_store_pb2.Execution,
                 metadata_store_pb2.Context],
-    name: Text,
-    is_custom_property: bool = False) -> Optional[Union[int, float, Text]]:
+    name: str,
+    is_custom_property: bool = False) -> Optional[Union[int, float, str]]:
   """Given a MLMD node and a (custom) property name, returns its value if any.
 
   Args:
@@ -337,9 +344,31 @@ def generate_model_card_for_model(
   return model_card
 
 
+def read_stats_protos(
+    stats_artifact_uri: str
+) -> List[statistics_pb2.DatasetFeatureStatisticsList]:
+  """Reads DatasetFeatureStatisticsList protos from provided uri.
+
+  Args:
+    stats_artifact_uri: the output artifact path of a StatsGen component.
+
+  Returns:
+    For each DatasetFeatureStatisticsList found in the directory, return in a
+    list.
+  """
+  stats_protos = []
+  for filename in tf.io.gfile.listdir(stats_artifact_uri):
+    if tf.io.gfile.isdir(os.path.join(stats_artifact_uri, filename)):
+      stats_proto = read_stats_proto(stats_artifact_uri, filename)
+      if stats_proto:
+        logging.info('Reading stats artifact from %s', filename)
+        stats_protos.append(stats_proto)
+  return stats_protos
+
+
 def read_stats_proto(
-    stats_artifact_uri: Text,
-    split: Text) -> Optional[statistics_pb2.DatasetFeatureStatisticsList]:
+    stats_artifact_uri: str,
+    split: str) -> Optional[statistics_pb2.DatasetFeatureStatisticsList]:
   """Reads DatasetFeatureStatisticsList proto from provided stats artifact uri.
 
   Args:
@@ -353,14 +382,14 @@ def read_stats_proto(
   stats = statistics_pb2.DatasetFeatureStatisticsList()
   feature_stats_path = os.path.join(stats_artifact_uri, split,
                                     'FeatureStats.pb')
-  stats_tfrecord_path = os.path.join(stats_artifact_uri,
-                                     split, 'stats_tfrecord')
+  stats_tfrecord_path = os.path.join(stats_artifact_uri, split,
+                                     'stats_tfrecord')
 
-  if os.path.exists(feature_stats_path):
+  if tf.io.gfile.exists(feature_stats_path):
     with tf.io.gfile.GFile(feature_stats_path, mode='rb') as f:
       stats.ParseFromString(f.read())
     return stats
-  elif os.path.exists(stats_tfrecord_path):
+  elif tf.io.gfile.exists(stats_tfrecord_path):
     serialized_stats = next(
         tf.compat.v1.io.tf_record_iterator(stats_tfrecord_path))
     stats.ParseFromString(serialized_stats)
@@ -372,8 +401,8 @@ def read_stats_proto(
 
 
 def read_metrics_eval_result(
-    metrics_artifact_uri: Text,
-    output_file_format: Optional[Text] = None) -> Optional[tfma.EvalResult]:
+    metrics_artifact_uri: str,
+    output_file_format: Optional[str] = None) -> Optional[tfma.EvalResult]:
   """Reads TFMA evaluation results from the evaluator output path.
 
   Args:
@@ -390,3 +419,183 @@ def read_metrics_eval_result(
     logging.warning('Cannot load eval results from: %s', metrics_artifact_uri)
     return None
   return result
+
+
+def annotate_eval_result_metrics(model_card: model_card_module.ModelCard,
+                                 eval_result: tfma.EvalResult):
+  """Annotates model_card's PerformanceMetrics for every metric in eval_result.
+
+  Args:
+    model_card: The model card object.
+    eval_result: A `tfma.EvalResult`.
+
+  Raises:
+    ValueError: if eval_result is improperly formatted.
+  """
+
+  def _parse_array_value(array: Dict[str, Any]) -> str:
+    data_type = array['dataType']
+    if data_type in _TYPE_FIELD_MAP:
+      type_field = _TYPE_FIELD_MAP[data_type]
+      return ', '.join([str(value) for value in array[type_field]])
+    else:
+      logging.warning('Received unexpected array %s', str(array))
+      return ''
+
+  for slice_repr, metrics_for_slice in (
+      eval_result.get_metrics_for_all_slices().items()):
+    # Parse the slice name
+    if not isinstance(slice_repr, tuple):
+      raise ValueError(
+          f'Expected EvalResult slices to be tuples; found {type(slice_repr)}')
+    slice_name = '_X_'.join(f'{a}_{b}' for a, b in slice_repr)
+    for metric_name, metric_value in metrics_for_slice.items():
+      # Parse the metric value
+      parsed_value = ''
+      if 'doubleValue' in metric_value:
+        parsed_value = metric_value['doubleValue']
+      elif 'boundedValue' in metric_value:
+        parsed_value = metric_value['boundedValue']['value']
+      elif 'arrayValue' in metric_value:
+        parsed_value = _parse_array_value(metric_value['arrayValue'])
+      else:
+        logging.warning(
+            'Expected doubleValue, boundedValue, or arrayValue; found %s',
+            metric_value.keys())
+      if parsed_value:
+        # Create the PerformanceMetric and append to the ModelCard
+        metric = model_card_module.PerformanceMetric(
+            type=metric_name, value=str(parsed_value), slice=slice_name)
+        model_card.quantitative_analysis.performance_metrics.append(metric)
+
+
+def filter_metrics(
+    eval_result: tfma.EvalResult,
+    metrics_include: Optional[List[Text]] = None,
+    metrics_exclude: Optional[List[Text]] = None) -> tfma.EvalResult:
+  """Filters metrics in a TFMA EvalResult.
+
+  Args:
+    eval_result: The TFMA EvalResult object.
+    metrics_include: The names of metrics to keep in the EvalResult. Mutually
+      exclusive with metrics_exclude.
+    metrics_exclude: The names of metrics to discard in the EvalResult. Mutually
+      exclusive with metrics_include.
+
+  Returns:
+    The eval_result with unwanted metrics filtered.
+
+  Raises:
+    ValueError: if both metrics_include and metrics_exclude are provided.
+  """
+  if metrics_include and not metrics_exclude:
+    include = lambda metric_name: metric_name in metrics_include
+  elif metrics_exclude and not metrics_include:
+    include = lambda metric_name: metric_name not in metrics_exclude
+  else:
+    raise ValueError('filter_metrics() requires exactly one of metrics_include '
+                     'and metrics_exclude.')
+
+  filtered_slicing_metrics = []
+  for slc, mtrc in eval_result.slicing_metrics:
+    filtered_mtrc = {}
+    for output_name in mtrc:
+      for subkey in mtrc[output_name]:
+        for mtrc_name in mtrc[output_name][subkey]:
+          if include(mtrc_name):
+            filtered_mtrc[output_name] = filtered_mtrc.get(output_name, {})
+            filtered_mtrc[output_name][subkey] = filtered_mtrc[output_name].get(
+                subkey, {})
+            filtered_mtrc[output_name][subkey][mtrc_name] = mtrc[output_name][
+                subkey][mtrc_name]
+    filtered_slicing_metrics.append(
+        tfma.view.SlicedMetrics(slice=slc, metrics=filtered_mtrc))
+
+  return tfma.EvalResult(
+      slicing_metrics=filtered_slicing_metrics,
+      plots=eval_result.plots,
+      attributions=eval_result.attributions,
+      config=eval_result.config,
+      data_location=eval_result.data_location,
+      file_format=eval_result.file_format,
+      model_location=eval_result.model_location)
+
+
+def filter_features(
+    dataset_stats: statistics_pb2.DatasetFeatureStatistics,
+    features_include: Optional[Sequence[Text]] = None,
+    features_exclude: Optional[Sequence[Text]] = None
+) -> statistics_pb2.DatasetFeatureStatistics:
+  """Filters features in a TFDV DatasetFeatureStatistics.
+
+  Args:
+    dataset_stats: The TFDV DatasetFeatureStatistics object.
+    features_include: The names or paths of features to keep. Mutually exclusive
+      with features_exclude.
+    features_exclude: The names or paths of features to discard. Mutually
+      exclusive with features_include.
+
+  Returns:
+    The DatasetFeatureStatisticsList with unwanted features filtered.
+
+  Raises:
+    ValueError: if both or neither of features_include and features_exclude are
+      provided.
+  """
+
+  # Check that inputs are valid, and create filter function
+  feature_name = lambda feature: feature.name or feature.path.step[0]
+  if features_include and not features_exclude:
+    include = lambda feature: feature_name(feature) in features_include
+  elif features_exclude and not features_include:
+    include = lambda feature: feature_name(feature) not in features_exclude
+  else:
+    raise ValueError('filter_features() requires exactly one of '
+                     'features_include and features_exclude.')
+
+  # Create new DatasetFeatureStatistics
+  filtered_data_stats = statistics_pb2.DatasetFeatureStatistics()
+  filtered_data_stats.CopyFrom(dataset_stats)
+
+  # Filter out features, and write to DatasetFeatureStatistics
+  filtered_features = [
+      feature for feature in dataset_stats.features if include(feature)
+  ]
+  del filtered_data_stats.features[:]
+  filtered_data_stats.features.extend(filtered_features)
+
+  # Return filtered DatasetFeatureStatistics
+  return filtered_data_stats
+
+
+def read_stats_protos_and_filter_features(
+    stats_artifact_uri: Text,
+    features_include: Optional[Sequence[Text]] = None,
+    features_exclude: Optional[List[Text]] = None
+) -> List[statistics_pb2.DatasetFeatureStatisticsList]:
+  """Reads DatasetFeatureStatisticsList protos and filters features.
+
+  Args:
+    stats_artifact_uri: the output artifact path of a StatsGen component.
+    features_include: The names or paths of features to keep. Mutually exclusive
+      with features_exclude.
+    features_exclude: The names or paths of features to discard. Mutually
+      exclusive with features_include.
+
+  Returns:
+    A list of DatasetFeatureStatisticsList from the provided path, with unwanted
+      features filtered.
+
+  Raises:
+    ValueError: if both or neither of features_include and features_exclude are
+      provided.
+  """
+  data_stats = read_stats_protos(stats_artifact_uri)
+  for dsfl in data_stats:
+    filtered_datasets = [
+        filter_features(dataset, features_include, features_exclude)
+        for dataset in dsfl.datasets
+    ]
+    del dsfl.datasets[:]
+    dsfl.datasets.extend(filtered_datasets)
+  return data_stats
